//
//  AmityEditMessageViewController.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 11/15/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import AmitySDK

protocol AmityEditMessageViewControllerDelegate: AnyObject {
    func amityEdit(_ viewController: AmityEditMessageViewController, willUpdateText text: String, onMessage message: AmityMessage?, metadata: [String: Any]?, mentionees: AmityMentioneesBuilder?)
}

final class AmityEditMessageViewController: UIViewController {

    @IBOutlet private weak var textView: UITextView!
    @IBOutlet private weak var updateButton: UIButton!
    
    @IBOutlet private weak var membersTableView: UITableView!
    @IBOutlet private weak var tableViewBottomContstraint: NSLayoutConstraint!
    
    private var mentionManager: AmityMentionManager?
    
    weak var delegate: AmityEditMessageViewControllerDelegate?
    
    private var message: AmityMessage?
    
    // To be injected.
    var client: AmityClient!
    
    // To be injected.
    var channelId: String!
    var subChannelId: String!
    lazy var channelPartecipation = AmityChannelParticipation(client: client,
                                                         andChannel: channelId)
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        updateButton.isEnabled = false
        updateButton.applyStyle(style: DefaultButtonStyle())
        
        textView.delegate = self
        textView.backgroundColor = UIColor.systemGroupedBackground
        textView.addRoundedCorner(radius: 8)
        let currentText = message?.data?["text"] as? String
        textView.text = currentText
        
        self.title = "Edit Message"
        
        let nib = UINib(nibName: SampleAppTableViewCell.identifier, bundle: nil)
        membersTableView?.register(nib, forCellReuseIdentifier: SampleAppTableViewCell.identifier)
        
        membersTableView?.tableFooterView = UIView()
        membersTableView.keyboardDismissMode = .none
        
        hideMentionsTableView()
        
        // Keyboard
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillChange), name: UIResponder.keyboardWillChangeFrameNotification, object: nil)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        channelPartecipation.startReading(subChannelId: subChannelId)
        
        setupMentionManager()
        textView.autocorrectionType = .no
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        channelPartecipation.stopReading(subChannelId: subChannelId)
    }
    
    private func setupMentionManager() {
        mentionManager = AmityMentionManager(withType: .message(channelId: channelId))
        mentionManager?.delegate = self

        if let metadata = message?.metadata {
            mentionManager?.setMentions(metadata: metadata, inText: message?.data?["text"] as? String ?? "")
        }
    }
    
    private func hideMentionsTableView() {
        membersTableView?.isHidden = true
    }
    
    func setMessage(message: AmityMessage) {
        setText(message: message)
    }
    
    private func setText(message: AmityMessage) {
        self.message = message
    }
    
    @IBAction func handleUpdateButton(_ sender: Any) {
        let metadata = mentionManager?.getMetadata()
        let mentionees = mentionManager?.getMentionees()
        
        delegate?.amityEdit(self, willUpdateText: textView.text, onMessage: message, metadata: metadata, mentionees: mentionees)
        
        mentionManager?.resetState()
        navigationController?.dismiss(animated: true, completion: nil)
    }
        
    static func make() -> UIViewController {
        let sb = UIStoryboard(name: "Chats", bundle: nil)
        return sb.instantiateViewController(identifier: "AmityEditMessageViewController")
    }

}

extension AmityEditMessageViewController: UITextViewDelegate {
    
    func textViewDidChange(_ textView: UITextView) {
        updateButton.isEnabled = textView.text != message?.data?["text"] as? String
    }
    
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        if (textView.text ?? "").count > AmityMentionManager.maximumCharacterCountForPost {
            showAlertForMaximumCharacters()
            return false
        }
        
        return mentionManager?.shouldChangeTextIn(textView, inRange: range, replacementText: text, currentText: textView.text ?? "") ?? true
    }
    
    func textViewDidChangeSelection(_ textView: UITextView) {
        mentionManager?.changeSelection(textView)
    }
}

extension AmityEditMessageViewController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        mentionManager?.addMention(from: textView, in: textView.text ?? "", at: indexPath)
    }
}

extension AmityEditMessageViewController: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return mentionManager?.itemsCount ?? 0
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let sampleAppCell = tableView.dequeueReusableCell(withIdentifier: SampleAppTableViewCell.identifier) as? SampleAppTableViewCell else { return UITableViewCell() }
        if let model = mentionManager?.item(at: indexPath) {
            sampleAppCell.titleLabel?.text = model.isChannel ? "@all: Notify everyone in this channel" : model.displayName
        }
        
        return sampleAppCell
    }

    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 40.0
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        if indexPath.row == (mentionManager?.itemsCount ?? 0) - 4 {
            mentionManager?.loadMore()
        }
    }
}

extension AmityEditMessageViewController {
    @objc private func keyboardWillChange(notification: Notification) {
        guard let endFrame = (notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue
        else { return }
        
        if endFrame.origin.y >= UIScreen.main.bounds.size.height {
            tableViewBottomContstraint.constant = 0
        } else {
            tableViewBottomContstraint.constant = endFrame.size.height
        }
    }

}

extension AmityEditMessageViewController: AmityMentionManagerDelegate {
    func didGetUsers(users: [AmityMentionUserModel]) {
        if users.isEmpty {
            membersTableView.isHidden = true
        } else {
            membersTableView.isHidden = false
            membersTableView.reloadData()
        }
    }

    func didCreateAttributedString(attributedString: NSAttributedString) {
        textView.attributedText = attributedString
    }

    func didMentionsReachToMaximumLimit() {
        let title = "Unable to mention"
        let message = "Mentions are reached to maximum limit"
        
        UIHelper.displayAlert(in: self, title: title, message: message)
    }

    func didCharactersReachToMaximumLimit() {
        showAlertForMaximumCharacters()
    }

    private func showAlertForMaximumCharacters() {
        let title = "Unable to mention"
        let message = "Text reached to maximum characters limit"
        
        UIHelper.displayAlert(in: self, title: title, message: message)
    }
}
